/*
 * bufrectpainter.h
 * 
 * Rectangle caching XPainter for performant rectangle drawing 
 * (single color). 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _INC_QTXLIB_BufferedRectPainter_H_
#define _INC_QTXLIB_BufferedRectPainter_H_ 1

#include "xpainter.h"

namespace QTX
{

// Class which allows you to draw rectangles via XDrawRectangles 
// and XFillRectangles using a rectangle cache. 
// There is exactly one XPainter per BufferedRectPainter. 
// The cache is automatically flushed if you draw more than it can 
// hold; you may also flush it explicitly. 
class BufferedRectPainter : public XPainter
{
	private:
		uint csize;    // size of cache 
		uint nr;       // number of used rectangles
		XRectangle *r;
		bool fill;
	public:
		// See Setup() for explanation on cache_size & fill args. 
		BufferedRectPainter(const QPaintDevice *pd,uint cache_size,bool fill=1);
		~BufferedRectPainter();
		
		uint GetCacheSize() const
			{  return(csize);  }
		
		// cache_size: number of XRectangles in cache. 
		// fill: 1 -> XFillRectangles
		//       0 -> XDrawRectangles
		// This allocates cache_size*sizeof(XRectangle) bytes. 
		// You cannot use cache_size=0. 
		void Setup(uint cache_size,bool fill=1);
		
		// Flush = draw the rects: 
		void Flush();
		
		// Discart cache content without flushing: 
		void Clear()
			{  nr=0;  }
		
		// Add a rect to the cache: 
		void AddRect(int x,int y,uint w,uint h)
		{
			if((!w || !h) && fill)  return;
			if(nr>=csize)  Flush();
			XRectangle *cr=&r[nr++];
			cr->x=x;  cr->y=y;  cr->width=w;  cr->height=h;
		}
};

}  // end of namespace QTX

#endif   /* _INC_QTXLIB_BufferedRectPainter_H_ */
