/*
 * bufrectpainterarray.h
 * 
 * Buffered rectangle painter array. Rectangle cache for performant 
 * rectangle drawing in different colors. 
 * 
 * Copyright (c) 2002 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file is part of QtXLib (Fast XLib drawing interface for Qt/X11). 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _INC_QTXLIB_BufferedRectPainterArray_H_
#define _INC_QTXLIB_BufferedRectPainterArray_H_ 1

#include "bufrectpainter.h"

namespace QTX
{

// This is a class which holds several BufferecdRectPainter classes. 
// This allows you e.g. to buffer rectangles with different colors. 
// NOTE: You have to explicitly set up each BufferedRectPainter in 
//       the array after calling the constructor or Setup() before 
//       you can actually draw. 
class BufferedRectPainterArray
{
	protected:
		uint asize;
		BufferedRectPainter **pa;
	public:
		// See Setup() for explanation on the arg. 
		// See also NOTE at the beginning of the class. 
		BufferedRectPainterArray(uint array_size)
			{  pa=NULL; asize=0;  Setup(array_size);  }
		~BufferedRectPainterArray()
			{  Setup(0);  }
		
		uint GetArraySize() const
			{  return(asize);  }
		
		// Get one BufferedRectPainter explicitly (to set the color, etc). 
		// This returns NULL if pidx is out of range or the corresponding 
		// BufferedRectPainter is not set up. 
		BufferedRectPainter *BRP(uint pidx)
			{  return(pidx>=asize ? NULL : pa[pidx]);  }
		BufferedRectPainter *operator[](uint pidx)
			{  return(BRP(pidx));  }
		
		// This sets up the BufferedRectPainter array holding 
		// array_size elements. 
		// Set array_size=0 to clear. 
		void Setup(uint array_size);
		
		// Set up a BufferedRectPainter with index pidx in the array. 
		// The args after pidx are passed to the constructor of 
		// the BufferedRectPainter. 
		// If the index idx is already set up, the old BufferedRectPainter 
		// is flushed and destroyed and the new one put in place. 
		// Return value: 0 -> OK; 1 -> pidx out of range 
		int Setup(uint pidx,
			const QPaintDevice *pd,uint cache_size,bool fill=true);
		int Setup(uint pidx,
			const QPainter *qp,uint cache_size,bool fill=true);
		
		// Flush all BufferedRectPainter classes: 
		void Flush();
		
		// Flush specified BufferedRectPainter: 
		// Return value: 
		//    0 -> OK; 
		//    1 -> pidx out of range. 
		//    2 -> painter not set up
		int Flush(uint pidx);
		
		// Add a rect to the correct BufferedRectPainter: 
		// pidx: painter index in range 0...array_size-1
		// Return value: 
		//    0 -> OK; 
		//    1 -> pidx out of range. 
		//    2 -> painter not set up
		int AddRect(uint pidx,int x,int y,uint w,uint h)
		{
			if(pidx>=asize)  return(1);
			BufferedRectPainter *ppa=pa[pidx];  if(!ppa)  return(2);
			ppa->AddRect(x,y,w,h);  return(0);
		}
};

}  // end of namespace QTX

#endif   /* _INC_QTXLIB_BufferedRectPainterArray_H_ */
